@@ -1,4 +1,4 @@
-# Copyright (C) 2017-2019 Pier Carlo Chiodi
+# Copyright (C) 2017-2020 Pier Carlo Chiodi
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -135,6 +135,13 @@ class ConfigParserGeneral(ConfigParserBase):
         f["transit_free"]["asns"] = ValidatorASNList(
             mandatory=False
         )
+        f["never_via_route_servers"] = OrderedDict()
+        f["never_via_route_servers"]["peering_db"] = ValidatorBool(
+            default=True
+        )
+        f["never_via_route_servers"]["asns"] = ValidatorASNList(
+            mandatory=False
+        )
 
         f["irrdb"] = OrderedDict()
         i = f["irrdb"]
@@ -196,7 +203,7 @@ class ConfigParserGeneral(ConfigParserBase):
         c["rpki_roas"] = OrderedDict()
         r = c["rpki_roas"]
         r["source"] = ValidatorOption("source",
-            ("ripe-rpki-validator-cache", "rtrlib"),
+            ("ripe-rpki-validator-cache", "rtr"),
             mandatory=True,
             default="ripe-rpki-validator-cache"
         )
