@@ -1,4 +1,4 @@
-# Copyright (C) 2017-2019 Pier Carlo Chiodi
+# Copyright (C) 2017-2020 Pier Carlo Chiodi
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,15 +25,6 @@ from pierky.arouteserver.ipaddresses import IPAddress, IPNetwork
 from pierky.arouteserver.tests.base import ARouteServerTestCase
 from pierky.arouteserver.tests.mocked_env import MockedEnv
 from pierky.arouteserver.tests.live_tests.instances import BGPSpeakerInstance
-from pierky.arouteserver.tests.live_tests.bird import BIRDInstanceIPv4, \
-                                                      BIRDInstanceIPv6
-from pierky.arouteserver.tests.live_tests.openbgpd import OpenBGPD60Instance, \
-                                                          OpenBGPD61Instance, \
-                                                          OpenBGPD62Instance, \
-                                                          OpenBGPD63Instance, \
-                                                          OpenBGPD64Instance, \
-                                                          OpenBGPD65Instance, \
-                                                          OpenBGPD65PortableInstance
 
 
 class LiveScenario(ARouteServerTestCase):
@@ -384,25 +375,10 @@ class LiveScenario(ARouteServerTestCase):
         else:
             _class = instance_or_class
 
-        if _class is BIRDInstanceIPv4 or \
-            _class is BIRDInstanceIPv6:
-            tag = "bird16"
-        elif _class is OpenBGPD60Instance:
-            tag = "openbgpd60"
-        elif _class is OpenBGPD61Instance:
-            tag = "openbgpd61"
-        elif _class is OpenBGPD62Instance:
-            tag = "openbgpd62"
-        elif _class is OpenBGPD63Instance:
-            tag = "openbgpd63"
-        elif _class is OpenBGPD64Instance:
-            tag = "openbgpd64"
-        elif _class is OpenBGPD65Instance:
-            tag = "openbgpd65"
-        elif _class is OpenBGPD65PortableInstance:
-            tag = "openbgpd65p"
-        else:
-            msg = "Unknown instance type: "
+        try:
+            tag = _class.TAG
+        except:
+            msg = "'TAG' missing for instance of type "
             if isinstance(instance_or_class, BGPSpeakerInstance):
                 msg += instance_or_class.name
             else:
@@ -617,7 +593,7 @@ class LiveScenario(ARouteServerTestCase):
             else:
                 reject_reasons = list(reject_reason)
             for code in reject_reasons:
-                assert code in range(1,15), "invalid reject_reason"
+                assert code in range(1,16), "invalid reject_reason"
 
         include_filtered = filtered if filtered is not None else False
         best_only = only_best if only_best is not None else False
